searchData={"items":[{"type":"module","title":"FeistelCipher.Migration","doc":"Migrations create functions FeistelCipher needs to function.","ref":"FeistelCipher.Migration.html"},{"type":"module","title":"Usage - FeistelCipher.Migration","doc":"To use migrations in your application you'll need to generate an `Ecto.Migration` that wraps\ncalls to `FeistelCipher.Migration`:\n\n```bash\nmix ecto.gen.migration add_feistel_cipher\n```\n\nOpen the generated migration in your editor and call the `up` and `down` functions on\n`FeistelCipher.Migration`:\n\n```elixir\ndefmodule MyApp.Repo.Migrations.AddFeistelCipher do\n  use Ecto.Migration\n\n  def up, do: FeistelCipher.Migration.up()\n\n  def down, do: FeistelCipher.Migration.down()\nend\n```\n\nThis will run all of FeistelCipher's versioned migrations for your database.\n\nNow, run the migration to create the table:\n\n```bash\nmix ecto.migrate\n```","ref":"FeistelCipher.Migration.html#module-usage"},{"type":"module","title":"Isolation with Prefixes - FeistelCipher.Migration","doc":"FeistelCipher supports namespacing through PostgreSQL schemas, also called \"prefixes\" in Ecto. With\nprefixes your jobs table can reside outside of your primary schema (usually public) and you can\nhave multiple separate job tables.\n\nTo use a prefix you first have to specify it within your migration:\n\n```elixir\ndefmodule MyApp.Repo.Migrations.AddPrefixedFeistelIdJobsTable do\n  use Ecto.Migration\n\n  def up, do: FeistelCipher.Migration.up(prefix: \"private\")\n\n  def down, do: FeistelCipher.Migration.down(prefix: \"private\")\nend\n```\n\nIn some cases, for example if your \"private\" schema already exists and your database user in\nproduction doesn't have permissions to create a new schema, trying to create the schema from the\nmigration will result in an error. In such situations, it may be useful to inhibit the creation\nof the \"private\" schema:\n\n```elixir\ndefmodule MyApp.Repo.Migrations.AddPrefixedFeistelIdJobsTable do\n  use Ecto.Migration\n\n  def up, do: FeistelCipher.Migration.up(prefix: \"private\", create_schema: false)\n\n  def down, do: FeistelCipher.Migration.down(prefix: \"private\")\nend\n```","ref":"FeistelCipher.Migration.html#module-isolation-with-prefixes"},{"type":"module","title":"Migrating Without Ecto - FeistelCipher.Migration","doc":"If your application uses something other than Ecto for migrations, be it an external system or\nanother ORM, it may be helpful to create plain SQL migrations for FeistelCipher database schema changes.\n\nThe simplest mechanism for obtaining the SQL changes is to create the migration locally and run\n`mix ecto.migrate --log-migrations-sql`. That will log all of the generated SQL, which you can\nthen paste into your migration system of choice.\n\nAlternatively, if you'd like a more automated approach, try using the [feistel_id_migations_sql][sql]\nproject to generate `up` and `down` SQL migrations for you.\n\n[sql]: https://github.com/btwb/feistel_id_migrations_sql","ref":"FeistelCipher.Migration.html#module-migrating-without-ecto"},{"type":"function","title":"FeistelCipher.Migration.down/1","doc":"Run the `down` changes.","ref":"FeistelCipher.Migration.html#down/1"},{"type":"function","title":"Example - FeistelCipher.Migration.down/1","doc":"Run migrations in an alternate prefix:\n\n    FeistelCipher.Migration.down(prefix: \"payments\")","ref":"FeistelCipher.Migration.html#down/1-example"},{"type":"function","title":"FeistelCipher.Migration.down_sql_for_table/2","doc":"Returns the SQL for dropping a trigger for a table to encrypt a `source` field to a `target` field.","ref":"FeistelCipher.Migration.html#down_sql_for_table/2"},{"type":"function","title":"Example - FeistelCipher.Migration.down_sql_for_table/2","doc":"FeistelCipher.Migration.down_sql_for_table(\"posts\", source: \"seq\", target: \"id\")","ref":"FeistelCipher.Migration.html#down_sql_for_table/2-example"},{"type":"function","title":"FeistelCipher.Migration.up/1","doc":"Run the `up` changes.","ref":"FeistelCipher.Migration.html#up/1"},{"type":"function","title":"Example - FeistelCipher.Migration.up/1","doc":"Run migrations in an alternate prefix:\n\n    FeistelCipher.Migration.up(prefix: \"payments\")","ref":"FeistelCipher.Migration.html#up/1-example"},{"type":"function","title":"FeistelCipher.Migration.up_sql_for_table/2","doc":"Returns the SQL for creating a trigger for a table to encrypt a `source` field to a `target` field.","ref":"FeistelCipher.Migration.html#up_sql_for_table/2"},{"type":"function","title":"Example - FeistelCipher.Migration.up_sql_for_table/2","doc":"FeistelCipher.Migration.up_sql_for_table(\"posts\", source: \"seq\", target: \"id\")","ref":"FeistelCipher.Migration.html#up_sql_for_table/2-example"},{"type":"task","title":"mix feistel_cipher.install","doc":"A Ecto migration for Feistel cipher","ref":"Mix.Tasks.FeistelCipher.Install.html"},{"type":"task","title":"Example - mix feistel_cipher.install","doc":"```bash\nmix feistel_cipher.install\n```\n\n* `--repo` or `-r` — Specify an Ecto repo for FeistelCipher to use.\n* `--prefix` or `-p` — Specify a prefix for the FeistelCipher schema, defaults to `public`\n* `--seed` or `-s` — Specify the seed for the Feistel cipher, should be less than 2^31, defaults to `1076943109`","ref":"Mix.Tasks.FeistelCipher.Install.html#module-example"},{"type":"extras","title":"FeistelCipher","doc":"# FeistelCipher\n\nDatabase functions for Feistel cipher.\n\nThis library provides Mix tasks and Ecto migrations to set up and manage Feistel cipher functions and triggers in your PostgreSQL database. Feistel cipher can be used to generate reversible, non-sequential IDs from sequential numbers.","ref":"readme.html"},{"type":"extras","title":"Installation - FeistelCipher","doc":"You can install FeistelCipher using `igniter` or by manually adding it to your dependencies.","ref":"readme.html#installation"},{"type":"extras","title":"Using igniter - FeistelCipher","doc":"```bash\nmix igniter.install feistel_cipher\n```","ref":"readme.html#using-igniter"},{"type":"extras","title":"Manual Installation - FeistelCipher","doc":"1.  Add `feistel_cipher` to your list of dependencies in `mix.exs`:\n\n    ```elixir\n    def deps do\n      [\n        {:feistel_cipher, \"~> 0.2.0\"}\n      ]\n    end\n    ```\n\n2.  Fetch the dependencies:\n\n    ```bash\n    mix deps.get\n    ```\n\n3.  Run the installer to generate the initial migration files:\n\n    ```bash\n    mix feistel_cipher.install\n    ```\n    This command will generate a migration file in your `priv/repo/migrations` directory. You can customize the installation with the following options:\n\n    * `--repo` or `-r`: Specify an Ecto repo for FeistelCipher to use.\n    * `--prefix` or `-p`: Specify a prefix for the FeistelCipher schema, defaults to `public`.\n    * `--seed` or `-p`: Specify the seed for the Feistel cipher, should be less than 2^31, defaults to `1_076_943_109`.","ref":"readme.html#manual-installation"},{"type":"extras","title":"Example Ecto Schema - FeistelCipher","doc":"Below is an example of an Ecto schema for a table that uses a Feistel cipher.\nIn this common setup:\n- The `seq` column is a `BIGSERIAL` (auto-incrementing integer) in the database, serving as the source for the Feistel cipher.\n- The `id` column is a `BIGINT` primary key, and its value is automatically generated by a database trigger using the `seq` column.\n\n```elixir\ndefmodule MyApp.MyTable do\n  use Ecto.Schema\n\n  # Defines `id` as the primary key (typically BIGINT in the DB).\n  # `autogenerate: true` indicates that its value is set by a database trigger\n  # based on the Feistel cipher of the `seq` column, not by a DB sequence directly on `id`.\n  @primary_key {:id, :id, autogenerate: true}\n  @foreign_key_type :id # For foreign keys referencing this table's `id`.\n\n  schema \"my_table\" do\n    # `seq` is the source for the Feistel cipher. In the database, this is typically a BIGSERIAL column.\n    # Ecto's `:id` type with `autogenerate: true` here reflects the auto-incrementing nature\n    # of a BIGSERIAL column when mapped in Ecto for a non-primary key field.\n    field :seq, :id, autogenerate: true\n\n    # The `id` field itself (the primary key) is implicitly defined by @primary_key.\n    # Its value will be populated by the Feistel cipher trigger.\n\n    timestamps()\n  end\nend\n```\n\nWhen creating the actual database table via a migration:\n- You would define `seq` as `BIGSERIAL`.\n- You would define `id` as `BIGINT` and set it as the primary key.\nThe Ecto schema above reflects how Ecto maps these PostgreSQL types and their behaviors. The `feistel_cipher` trigger will take the auto-generated `seq` value and populate the `id` column with the Feistel-ciphered value.","ref":"readme.html#example-ecto-schema"},{"type":"extras","title":"Creating Triggers for Tables - FeistelCipher","doc":"To automatically encrypt a source column into a target column for a specific table, you can create a database trigger. The `FeistelCipher.Migration.up_sql_for_table/2` function helps generate the necessary SQL.\n\nFor example, if you have a table named `posts` with a `seq` column (source) and you want to store the Feistel ciphered ID in an `id` column (target):\n\n```elixir\n# Example: priv/repo/migrations/YYYYMMDDHHMMSS_add_posts_feistel_trigger.exs\ndefmodule MyApp.Repo.Migrations.AddPostsFeistelTrigger do\n  use Ecto.Migration\n\n  def up do\n    # Default bits: 62\n    execute FeistelCipher.Migration.up_sql_for_table(\"posts\", bits: 40, source: \"seq\", target: \"id\")\n  end\n\n  def down do\n    execute FeistelCipher.Migration.down_sql_for_table(\"posts\", bits: 40, source: \"seq\", target: \"id\")\n  end\nend\n```\n\nThen run `mix ecto.migrate` to apply the migration.\n\nThe `up_sql_for_table/2` function accepts the following options:\n\n*   `table`: (String, required) The name of the table.\n*   `source`: (String, required) The name of the source column containing the `bigint` integer (typically from a `BIGSERIAL` column like `seq`).\n*   `target`: (String, required) The name of the target column to store the encrypted integer (typically the `BIGINT` primary key like `id`).\n*   `bits`: (Integer, optional) The number of bits for the Feistel cipher. Must be an even number, 62 or less. Defaults to `62`.\n\nThe trigger will automatically populate the `target` column when a new row is inserted. It also prevents manual modification of the `target` column on update.","ref":"readme.html#creating-triggers-for-tables"},{"type":"extras","title":"License - FeistelCipher","doc":"MIT","ref":"readme.html#license"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}